@@ -1,10 +1,13 @@
+# -*- coding: utf-8 -*-
+
+import os
+
+import pandas as pd
+import pytest
+from numpy.testing import assert_array_almost_equal as equal
 
 import pypsa
-import pytest
-import pandas as pd
-import sys
-import os
-from numpy.testing import assert_array_almost_equal as equal
+
 
 @pytest.fixture
 def target_gen_p():
@@ -15,7 +18,7 @@ def target_gen_p():
         "opf-storage-hvdc",
         "opf-storage-data",
         "results",
-        "generators-p.csv"
+        "generators-p.csv",
     )
     return pd.read_csv(target_path, index_col=0, parse_dates=True)
 
@@ -27,25 +30,17 @@ def network():
         "..",
         "examples",
         "opf-storage-hvdc",
-        "opf-storage-data"
+        "opf-storage-data",
     )
     return pypsa.Network(csv_folder)
 
 
 def test_opf_pyomo(network, target_gen_p):
-    network.lopf(solver_name='glpk', pyomo=True)
-    equal(
-        network.generators_t.p.reindex_like(target_gen_p),
-        target_gen_p,
-        decimal=2
-    )
+    network.lopf(solver_name="glpk", pyomo=True)
+    equal(network.generators_t.p.reindex_like(target_gen_p), target_gen_p, decimal=2)
 
 
 def test_opf_lowmem(network, target_gen_p):
-    status, _ = network.lopf(solver_name='glpk', pyomo=False)
-    assert status == 'ok'
-    equal(
-        network.generators_t.p.reindex_like(target_gen_p),
-        target_gen_p,
-        decimal=2
-    )
+    status, _ = network.lopf(solver_name="glpk", pyomo=False)
+    assert status == "ok"
+    equal(network.generators_t.p.reindex_like(target_gen_p), target_gen_p, decimal=2)
